# -*- coding:utf-8 -*-
# src/jd_parse.py


__all__ = [
    'JdParse'
]


class JdParse(object):

    @staticmethod
    def remove_comment(s):
        # 去除 s 字符串中的 注释： /**/，支持 一行中有多个注释
        r = ''
        begin = 0
        p = 0
        len_max = len(s)

        while p < len_max:
            while p < len_max and s[p] != r'/':     # 找到 字符 '/'
                p += 1
            if p >= len_max:
                break

            p += 1
            if p >= len_max:
                break

            if s[p - 1] != r'/':
                p += 1
                continue

            if s[p] == '*':     # 如果 '/' 后面的字符 是 '*'，说明为 注释开始
                r += s[begin: p - 1]

                p += 1
                end = False
                while not end and p < len_max:
                    while p < len_max and s[p] != '*':      # 继续找 注释结尾 '*/'
                        p += 1
                    if p >= len_max:
                        break

                    if s[p] != '*':
                        p += 1
                        continue

                    p += 1
                    if p >= len_max:
                        break

                    if s[p] == r'/':
                        p += 1
                        begin = p
                        end = True
            else:
                r += s[begin: p]
                begin = p
        r += s[begin: len_max]

        return r



# -*- coding: utf-8 -*-
from enum import Enum, unique
import json
import requests
from urllib3.exceptions import MaxRetryError, NewConnectionError
import base64
from cryptography.exceptions import InvalidTag
from cryptography.hazmat.primitives.ciphers.aead import AESGCM

__version__ = '2.9'
__all__ = ['RequestType', 'Core', 'aes_decrypt']


@unique
class RequestType(Enum):
    GET = 'GET'
    POST = 'POST'
    PATCH = 'PATCH'
    PUT = 'PUT'
    DELETE = 'DELETE'


_timeout = (3.05, 27)   # 超时时间, 连接超时 和 读取超时
# _timeout = None


class Core:
    def __init__(self,
                 api_key=None,      # 秘钥 key
                 logger=None,       # 日志
                 proxy=None         # 代理
                 ):
        self._proxy = proxy
        self._api_key = api_key
        self.gate_way = ''
        self._logger = logger

    def request(self, path, method=RequestType.GET, data=None, files=None, headers=None,
                show_url=True, show_response=True):
        if headers is None:
            headers = {}
        if files:
            headers.update({'Content-Type': 'multipart/form-data'})
        else:
            headers.update({'Content-Type': 'application/json'})
        headers.update({'Accept': 'application/json'})
        if self._logger:
            if show_url:
                self._logger.debug('Request url: %s' % self.gate_way + path)
            self._logger.debug('Request type: %s' % method.value)
            self._logger.debug('Request headers: %s' % headers)
            self._logger.debug('Request params: %s' % data)
        try:
            if method == RequestType.GET:
                response = requests.get(url=self.gate_way + path, headers=headers, proxies=self._proxy,
                                        timeout=_timeout)
            elif method == RequestType.POST:
                response = requests.post(url=self.gate_way + path, json=None if files else data,
                                         data=data if files else None, headers=headers, files=files,
                                         proxies=self._proxy, timeout=_timeout)
            elif method == RequestType.PATCH:
                response = requests.patch(url=self.gate_way + path, json=data, headers=headers, proxies=self._proxy,
                                          timeout=_timeout)
            elif method == RequestType.PUT:
                response = requests.put(url=self.gate_way + path,  json=data, headers=headers, proxies=self._proxy,
                                        timeout=_timeout)
            elif method == RequestType.DELETE:
                response = requests.delete(url=self.gate_way+path, headers=headers, proxies=self._proxy,
                                           timeout=_timeout)
            else:
                raise Exception('sdk does no support this request type.')
        except (TimeoutError, requests.exceptions.ConnectionError, MaxRetryError, NewConnectionError):
            return 500, {'code': 500, 'msg': 'Timeout Error'}
        if self._logger:
            self._logger.debug('Response status code: %s' % response.status_code)
            self._logger.debug('Response headers: %s' % response.headers)
            if show_response:
                self._logger.debug('Response content: %s' % response.text)
        if response.status_code == 404:
            return response.status_code, {'code': response.status_code, 'msg': response.text}
        elif response.status_code in [405]:
            return response.status_code, {'code': response.status_code, 'msg': 'Method Not Allowed'}
        content_type = response.headers.get('Content-Type')
        message = response.text if content_type and 'application/json' in content_type else response.content
        try:
            if isinstance(message, str):
                message = json.loads(message)
            elif isinstance(message, bytes):
                message = json.loads(message.decode("utf-8"))
        except json.decoder.JSONDecodeError:
            message = {}
        return response.status_code, message

    def decrypt_callback(self, headers, body):
        """
        通知的数据会加密 post 到对接方接口中，由于涉及到回调加密和解密，接入方必须先通过微校商务侧申请
        通知秘钥 key，申请好通知秘钥 key 后才能解密回调通知
        :param headers:
        :param body:
        :return:
        """
        if isinstance(body, bytes):
            body = body.decode('UTF-8')
        if self._logger:
            self._logger.debug('Callback Header: %s' % headers)
            self._logger.debug('Callback Body: %s' % body)
        data = json.loads(body)
        resource_type = data.get('resource_type')
        if resource_type != 'encrypt-resource':
            return None
        resource = data.get('resource')
        if not resource:
            return None
        algorithm = resource.get('algorithm')
        if algorithm != 'AEAD_AES_256_GCM':
            raise Exception('sdk does not support this algorithm')
        nonce = resource.get('nonce')
        ciphertext = resource.get('ciphertext')
        associated_data = resource.get('associated_data')
        if not (nonce and ciphertext):
            return None
        if not associated_data:
            associated_data = ''
        result = aes_decrypt(
            nonce=nonce,
            ciphertext=ciphertext,
            associated_data=associated_data,
            notify_key=self._api_key)
        if self._logger:
            self._logger.debug('Callback resource: %s' % result)
        return result


def aes_decrypt(nonce, ciphertext, associated_data, notify_key):
    """
    微卡 订单流水实时推送第三方平台， 数据解密

    下面详细描述对通知数据进行解密的流程：
    1. 通知秘钥 key，记为 notify_key；
    2. 针对 resource.algorithm 中描述的算法（目前为 AEAD_AES_256_GCM），取得对应的参数 nonce 和 associated_data；
    3. 使用 notify_key、nonce 和 associated_data，对数据密文 resource.ciphertext 进行解密，得到 JSON 形式的资源
    对象（资源对象就是具体的数据内容）；
    注： AEAD_AES_256_GCM 算法的接口细节，请参考 rfc5116。微校支付使用的通知密钥 key 长度为 32 个字节，随机串 nonce
    长度 32 个字节，associated_data 长度小于 16 个字节并可能为空。
    :param nonce:               加密使用的随机串
    :param ciphertext:          Base64编码后的数据密文
    :param associated_data:     附加数据
    :param notify_key:          秘钥
    :return:
    """
    key_bytes = notify_key.encode('UTF-8')
    nonce_bytes = nonce.encode('UTF-8')
    associated_data_bytes = associated_data.encode('UTF-8') if associated_data else None
    data = base64.b64decode(ciphertext)
    aes_gcm = AESGCM(key=key_bytes)
    try:
        result = aes_gcm.decrypt(nonce=nonce_bytes, data=data, associated_data=associated_data_bytes).decode('UTF-8')
    except InvalidTag:
        result = None
    return result



# -*- coding:utf-8 -*-
# 文件:  errcode.py

ERR_DATE_FMT = 200
ERR_DATE_FMT_MSG = '“%s”格式错误【%s】，期望 年-月-日！'
E_PUB_NUM = 706
E_PUB_NUM_MSG = '【%s】格式错误，请输入数值！'
E_PUB_NUM_INT = 707
E_PUB_NUM_INT_MSG = '【%s】格式错误，请输入整数！'
E_PUB_NUM_PT2 = 708
E_PUB_NUM_PT2_MSG = '【%s】最多两位小数！'
E_PUB_NEED = 709
E_PUB_NEED_MSG = '请输入【%s】!'
E_PUB_CUSTOMIZED_ERR = 819
E_PUB_CUSTOMIZED_ERR_MSG = '自定义错误提示，通过ppt设置'



# -*- coding:utf-8 -*-
# 文件:  aes.py
# 日期:  2022/9/8 9:37
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from cryptography.exceptions import InvalidTag
import base64
import binascii
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend


__version__ = "1.2"
__all__ = [
    "aes_gcm_encrypt", "aes_gcm_decrypt",
    "aes_cbc_encrypt",
    "aes_ecb_encrypt", "aes_ecb_decrypt"
]


def aes_gcm_encrypt(key, data, nonce, associated_data=None):
    """
    AES-GCM加密
    :param key: 密钥。16, 24 or 32字符长度的字符串
    :param data: 待加密字符串
    :param associated_data: 附加数据，一般为None
    :param nonce: 随机值，和MD5的“加盐”有些类似，目的是防止同样的明文块，始终加密成同样的密文块
    """
    key = key.encode("utf-8")
    data = data.encode("utf-8")
    nonce = nonce.encode("utf-8") if nonce else "12345678".encode("utf-8")
    associated_data = associated_data.encode("utf-8") if associated_data else None

    # 生成加密器
    cipher = AESGCM(key)
    # 加密数据
    crypt_bytes = cipher.encrypt(nonce, data, associated_data)
    return base64.b64encode(crypt_bytes).decode()


def aes_gcm_decrypt(key, cipher_data, nonce, associated_data=None):
    """
    AES-GCM解密
    :param key: 密钥。16, 24 or 32字符长度的字符串
    :param cipher_data: encrypt_aes_gcm 方法返回的数据
    :param nonce: 随机值，和MD5的“加盐”有些类似，目的是防止同样的明文块，始终加密成同样的密文块
    :param associated_data: 附加数据，一般为None
    :return:
    """
    key = key.encode("utf-8")
    nonce = nonce.encode("utf-8") if nonce else "12345678".encode("utf-8")
    associated_data = associated_data.encode("utf-8") if associated_data else None
    cipher_data = base64.b64decode(cipher_data)  # 进行base64解码
    cipher = AESGCM(key)
    # 解密数据
    try:
        plaintext = cipher.decrypt(nonce, cipher_data, associated_data).decode("utf-8")
    except InvalidTag:
        plaintext = None
    return plaintext


block_size = 128


def aes_ecb_encrypt(secret_key, data):
    """AES_ECB加密数据
    :param secret_key: 加密秘钥
    :param data: 需要加密数据
    """
    # 将数据转换为byte类型
    data = data.encode("utf-8")
    secret_key = secret_key.encode("utf-8")

    # 填充数据采用 pkcs7
    pad = padding.PKCS7(block_size).padder()
    pad_data = pad.update(data) + pad.finalize()

    # 创建密码器
    cipher = Cipher(algorithms.AES(secret_key), mode=modes.ECB(), backend=default_backend())
    # 加密数据
    encryptor = cipher.encryptor()
    encrypted_data = encryptor.update(pad_data)
    return base64.b64encode(encrypted_data).decode()


def aes_ecb_decrypt(secret_key, data):
    """AES_ECB解密数据"""
    secret_key = secret_key.encode("utf-8")
    data = base64.b64decode(data)

    # 创建密码器
    cipher = Cipher(algorithms.AES(secret_key), mode=modes.ECB(), backend=default_backend())
    decrypter = cipher.decryptor()
    decrypt_data = decrypter.update(data)
    un_pad = padding.PKCS7(block_size).unpadder()
    un_pad_decrypt_data = un_pad.update(decrypt_data) + un_pad.finalize()
    return un_pad_decrypt_data.decode("utf-8")


def _zero_pad(s):
    # Zero Padding
    if isinstance(s, bytes):
        s = s.decode("utf-8")
    return (s + (16 - len(s) % 16) * '\0').encode('utf-8')


def aes_cbc_encrypt(data, key, iv, encode=1):
    """
    use AES CBC to encrypt message, using key and init vector， padding: ZeroPadding
    :param data: 待加密的数据
    :param key:  加密所用的key
    :param iv:   加密所有的初始化向量 Init Vector
    :param encode:  加密后密文的编码方式， 1-base64, 2-binascii.hexlify(转为16进制,1byte->2字符)
    :return:
    """
    padded_data = _zero_pad(data)   # bytes
    key = key.encode('utf8')
    iv = iv[:16].encode('utf8')
    backend = default_backend()
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv), backend=backend)
    encryptor = cipher.encryptor()
    enc_content = encryptor.update(padded_data) + encryptor.finalize()
    if encode == 1:  # base64编码
        enc_data = base64.b64encode(enc_content).decode('utf8')
    else:   # encode == 2  # 1byte 转 16进制2个字符,
        enc_data = binascii.hexlify(enc_content).decode('utf-8')
    return enc_data


if __name__ == '__main__':
    my_key = "JP67DD47BSO8B2C43JEZ7UA0ZHV60SB2"
    my_iv = "1234567812345678"
    my_data = '{"lang":"zh-CN","pageNumber":1,"pageSize":10,"cycleId":"1522973936269266945"}'

    print("原始数据：%s" % my_data)
    r = aes_ecb_encrypt(my_key, my_data)
    print("加密数据 [aes_ecb_encrypt]：%s" % r)
    r = aes_ecb_decrypt(my_key, r)
    print("解密数据：%s" % r)

    ret = aes_cbc_encrypt(my_data, my_key, my_iv, encode=1)
    print(f"加密后 [aes_cbc_encrypt]：{ret}")



# -*- coding: utf8 -*-
# 文件:  mini_app_template.py
# 日期:  2022/1/22 17:55
from .tools import JdTools
from .logs import logger


__all__ = ['template_dict', 'get_template', "JdMiniAppTemplate", "jd_parse_template"]
_author__ = 'WXG'
__version__ = '1.2'


template_dict = {}  # template id: (编号, 模板ID, 标题)


def get_template(temp_id):
    # 根据模板值，获取模板ID
    return template_dict.get(temp_id, (0, 0))[1]


def jd_parse_template(template_file):
    with open(template_file, 'r', encoding='utf8') as f:
        content = f.read()

        size = len(content)
        i = 0
        while i < size:

            s, i = get_template_section(content, i)
            if i == -1:
                break

            i += 3  # 跳过 ``` 的长度
            if 'type' not in s:
                continue

            key = single_info(s, 'type')
            code = single_info(s, '编号')
            template_id = single_info(s, '模版ID')
            if not template_id:
                template_id = single_info(s, '模板ID')
            template_name = single_info(s, '标题')
            if not template_name:
                template_name = single_info(s, '模版标题')
            if key:
                template_dict[int(key)] = (code, template_id, template_name)


def get_template_section(content, start):
    # 从 内容 content 的 start 起始位置(0 开始), 查询 ``` 和 ``` 之间的内容
    p1 = content.find('```', start)
    if p1 == -1:
        return '', -1
    p2 = content.find('```', p1 + 3)
    return content[p1 + 3: p2], p2


def single_info(template_msg, key):
    p1 = template_msg.find(key)
    if p1 == -1:
        return ''
    k = p1 + len(key)
    p2 = template_msg.find('\n', k)
    line = template_msg[k: p2]
    line = line.replace('=', '').replace('：', '').replace(':', '')
    line = JdTools.trim(line)
    return line


class JdMiniAppTemplate(object):

    def __init__(self):
        self._show_template = True

    def set_show_template(self, is_show):
        self._show_template = is_show

    def show(self, force_show=False):
        if self._show_template or force_show:
            logger.debug(f'微信模板 读取完成！ 数量：{len(template_dict)}')
            for m in template_dict.items():
                logger.debug(f'{m}')



# -*- coding: utf8 -*-
# 文件:  ip.py
# 日期:  2021/10/7 13:41
import platform
import socket

__version__ = "1.1"
__all__ = ['get_ip', 'is_windows_os', 'get_hostname']


def get_ip():
    if is_windows_os():
        hostname = socket.gethostname()
        ip = socket.gethostbyname(hostname)
    else:
        ip = '0.0.0.0'
    return ip


def get_hostname():
    hostname = socket.gethostname()
    return hostname


def is_windows_os():
    return 'Windows' in platform.system()



# -*- coding:utf-8 -*-
# 文件:  gen_key.py
# 日期:  2022/9/8 12:01
import random
import string

__version__ = '1.0'
__all__ = ['jd_create_key', 'get_nonce']


def jd_create_key(size=32, start=2):
    """
    生成随机 key
    :param size:    长度，默认为32个字符
    :param start:   开始字符，1-数字， 2-字母
    :return:  只包含大写字母和数字，且字母 和 数字交替出现
    """
    arr = []
    max_letter = 3
    cur_max = random.randint(1, max_letter)
    cnt = 0
    who = start  # 1-数字， 2-字母
    while len(arr) < size:
        if who == 1:
            i = random.randint(0, 9)
            arr.append(str(i))
        else:
            i = random.randint(0, 25)
            arr.append(chr(65 + i))
        cnt += 1
        if cnt >= cur_max:
            cnt = 0
            cur_max = random.randint(1, max_letter)
            who = 1 if who == 2 else 2
    
    return "".join(arr)


def get_nonce(size=16):
    """
    生成随机字符串
    :param size:    字符串长度默认为16,
    :return:  字符包括大小写字母 和 数字
    """
    nonce = ''.join(random.sample(string.ascii_letters + string.digits, size))
    return nonce


if __name__ == '__main__':
    print(jd_create_key())



# -*- coding:utf-8 -*-
# 文件:  operate_file.py
# 日期:  2021/11/17 14:49
import os
import zipfile
import shutil
from .logs import logger

__version__ = '1.0.5'
__all__ = ['move_file', 'copy_file', 'zip_dir', 'make_dirs']


def move_file(src: str, dst: str):
    if not os.path.isfile(src):
        logger.error("move_file src file not exist [%s]!" % src)
        return False
    else:
        filepath, filename = os.path.split(dst)    # 分离文件名和路径
        if not os.path.exists(filepath):
            os.makedirs(filepath)   # 创建路径
        shutil.move(src, dst)   # 移动文件
        # logger.debug("move %s -> %s" % (src, dst))
        return True


def copy_file(src: str, dst: str):
    if not os.path.isfile(src):
        logger.error("copy_file src file not exist [%s]!" % src)
        return False
    else:
        filepath, filename = os.path.split(dst)    # 分离文件名和路径
        if not os.path.exists(filepath):
            os.makedirs(filepath)   # 创建路径
        shutil.copyfile(src, dst)   # 复制文件
        # logger.debug("copy %s -> %s" % (src, dst))
        return True


def zip_dir(dir_path, out_full_name):
    """
    压缩指定文件夹
    :param dir_path: 目标文件夹路径
    :param out_full_name: 压缩文件保存路径+xxx.zip
    :return: 无
    """
    z = zipfile.ZipFile(out_full_name, "w", zipfile.ZIP_DEFLATED)
    for path, dir_names, filenames in os.walk(dir_path):
        # 去掉目标跟路径，只对目标文件夹下边的文件及文件夹进行压缩
        f_path = path.replace(dir_path, '')
        for filename in filenames:
            z.write(os.path.join(path, filename), os.path.join(f_path, filename))
    z.close()


def make_dirs(full_path, last_file=False):
    if last_file:
        pathname = full_path.rsplit('/')[0]
    else:
        pathname = full_path
    if not os.path.exists(pathname):
        os.makedirs(pathname)



# -*- coding:utf-8 -*-
# utils/measure.py
# ---------------------------------------------------------
# 时间测量， 用于测试 函数功能耗时长短
#  2020-1-12
# 用法
#     time_measure = TimeMeasure('dance_check_in_report_get')
#     time_measure.measure('begin')
#
#     time_measure.measure('t1')
#     被测试函数
#     time_measure.measure('t2')
#     time_measure.show_time()      # 显示 t1 到 t2 的时间
#
#     time_measure.measure('end')
#     time_measure.show('end', 'begin')     # 显示 begin 到 end 的时间
# ---------------------------------------------------------
import time
import functools
from .logs import logger

__version__ = "1.2"
__author__ = ""
__all__ = [
    'TimeMeasure', 'timeit'
]

LOGGER = logger


class TimeMeasure(object):
    def __init__(self, module=''):
        self.module = module
        self.time_line = {}     # name: time
        self.out = True
        self.name_list = []

    def measure(self, name):
        tm = time.time()
        self.time_line[name] = tm
        self.name_list.append((name, tm))

    def show(self, pt1, pt2):
        if not self.out:
            return
        t1 = self.time_line.get(pt1)
        t2 = self.time_line.get(pt2)
        if t1 is None or t2 is None:
            return
        msg = '[%s] time elapse [%s - %s]: %s' % (self.module, pt1, pt2, abs(t1-t2))
        LOGGER.debug(msg)

    def show_time(self):
        if not self.out:
            return
        len_name = len(self.name_list)
        if len_name < 2:
            return
        n1, t1 = self.name_list[len_name - 1]
        n2, t2 = self.name_list[len_name - 2]
        msg = '[%s] time elapse [%s - %s]: %s' % (self.module, n1, n2, abs(t1 - t2))
        LOGGER.info(msg)


def timeit(desc=''):
    def timed(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            t1 = time.time()
            result = f(*args, **kwargs)
            t2 = time.time()
            LOGGER.debug(f'[{desc} {f.__name__}] time: {t2-t1}s')
            return result
        return wrapper
    return timed



__version__ = '0.0.7'

from .logs import *
from .tools import *
from .core import *
from .crypto import *
from .utils import *
from .parser_config import JdIniReader
from .cfg import *
from .jd_ip import *
from .measure import *
from .oper_file import *
from .rsa_pwd import *
from .cmd import *
from .mini_app_template import *
from .jd_parser import JdParse
from .param_check import JdCheck



# -*- coding: utf-8 -*-
import datetime
import json
from base64 import b64decode
from cryptography.exceptions import InvalidTag
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
"""
DATE        AUTHOR  DESCRIPTION
----------  ------  -----------------------------------------------------------------
2022-07-02  WXG     添加微信支付通知解密函数 notify_decrypt , 版本改为 1.3
"""

__version__ = "1.3"
__all__ = [
    "generate_map", "generate_map_ex", "cyt_map", "gen_key",
    "notify_decrypt"
]


def generate_map(records, field, add_columns=True):
    """
    将 子表记录，根据主表id，缓存到 map表  1:N
    :param records:     子表记录 列表
    :param field:       主表记录 id 字段名。通过 getattr 访问，主表
    :param add_columns:    子表 在 records 中位置， True， 取记录r[0]， False 取记录r
    :return:  records_map  缓存 子表记录的 map 表
    """
    records_map = dict()
    for i in records:
        if add_columns:
            r = i[0]
        else:
            r = i
        id_ = getattr(r, field)
        if id_ in records_map:
            records_map[id_].append(i)
        else:
            records_map[id_] = [i]
    return records_map


def generate_map_ex(records, add_columns=True, *args):
    """
    将 记录，根据 主表id，缓存到 map表   1:1
    :param records:         记录 列表
    :param add_columns:     记录中是否使用了add_columns方法， True-是， False-否
    :param args:            记录 关键字 key，多个使用 连字符拼接。 通过 getattr 访问
    :return:  records_map  记录的 map 表， key 中 value 唯一
    """
    records_map = dict()
    for i in records:
        if add_columns:
            r = i[0]
        else:
            r = i

        ids = []
        for key_ in args:
            ids.append(getattr(r, key_))

        if len(ids) == 1:   # 只有一个key，保留原来的 int 类型
            id_ = ids[0]
        else:   # 多个key， 转为字符串
            id_ = '-'.join([str(i) for i in ids])
        records_map[id_] = i
    return records_map


def gen_key(*args):
    """生成key， args 为 构造key 的字段值"""
    ids = []
    for key_ in args:
        ids.append(key_)

    if len(ids) == 1:  # 只有一个key，保留原来的 int 类型
        id_ = ids[0]
    else:  # 多个key， 转为字符串
        id_ = '-'.join([str(i) for i in ids])
    return id_


def cyt_map(records, one_to_one, *args, attr=True):
    """将记录，根据 args 提供的 key，缓存到 map表
    :param records:     记录
    :param one_to_one:  1:1 or 1:N
    :param args:        记录 关键字 key，多个使用 连字符拼接， 通过 records[].key 可以访问
    :param attr:        True 可以通过 getattr 获取属性； False 通过 [key] 获取属性
    :return:  records_map  记录map, 1：N
    """
    records_map = dict()
    for i in records:
        ids = []
        for key_ in args:
            if attr:
                val = getattr(i, key_)
            else:
                val = i[key_]

            if isinstance(val, datetime.datetime):
                val = val.date()
            ids.append(val)

        if len(ids) == 1:  # 只有一个key，保留原来的 int 类型
            id_ = ids[0]
        else:  # 多个key， 转为字符串
            id_ = '-'.join([str(i) for i in ids])
        if one_to_one:
            records_map[id_] = i    # 1:1 模式   k: v
        else:
            if id_ in records_map:
                records_map[id_].append(i)   # 1:N 模式   k: [v1, v2, ...]
            else:
                records_map[id_] = [i]
    return records_map


def notify_decrypt(nonce: str, ciphertext: str, associated_data: str, key: str) -> dict:
    """
    通知报文
    通知的数据会加密 post 到对接方接口中，由于涉及到回调加密和解密，接入方必须先通过微校商务侧申请通知秘钥 key，
    申请好通知秘钥 key 后才能解密回调通知
    original_type 原始回调类型，为transaction

    参数解密

    通知秘钥 key，记为 notify_key；
    针对 resource.algorithm 中描述的算法（目前为 AEAD_AES_256_GCM），取得对应的参数 nonce 和 associated_data；
    使用 notify_key、nonce 和 associated_data，对数据密文 resource.ciphertext 进行解密，得到 JSON 形式的
    资源对象（资源对象就是具体的数据内容）；
    注： AEAD_AES_256_GCM 算法的接口细节，请参考 rfc5116。微校支付使用的通知密钥 key 长度为 32 个字节，随机串 nonce
    长度 32 个字节，associated_data 长度小于 16 个字节并可能为空。
    参考： https://wiki.weixiao.qq.com/api/third/withhold/payRunning.html

    https://pay.weixin.qq.com/wiki/doc/apiv3_partner/apis/chapter4_5_5.shtml
    参数解密
    下面详细描述对通知数据进行解密的流程：
    1、用商户平台上设置的APIv3密钥【微信商户平台—>账户设置—>API安全—>设置APIv3密钥】，记为key；
    2、针对resource.algorithm中描述的算法（目前为AEAD_AES_256_GCM），取得对应的参数nonce和associated_data；
    3、使用key、nonce和associated_data，对数据密文resource.ciphertext进行解密，得到JSON形式的资源对象；
    注： AEAD_AES_256_GCM算法的接口细节，请参考rfc5116。微信支付使用的密钥key长度为32个字节，随机串nonce长度12个字节，
    associated_data长度小于16个字节并可能为空字符串。

    :param nonce:           随机串
    :param ciphertext:      密文
    :param associated_data: 附加数据
    :param key:             秘钥 key
    :return:  字典对象，解密失败时，返回 空 字典 dict
    """
    key_bytes = key.encode('UTF-8')
    nonce_bytes = nonce.encode('UTF-8')
    associated_data_bytes = associated_data.encode('UTF-8')
    data = b64decode(ciphertext)
    aes_gcm = AESGCM(key=key_bytes)
    try:
        result = aes_gcm.decrypt(nonce=nonce_bytes, data=data, associated_data=associated_data_bytes).decode('UTF-8')
    except InvalidTag:
        result = ''  # 解密失败
    if result:
        data = json.loads(result)
    else:
        data = dict()
    return data



# -*- coding:utf-8 -*-
# 文件:  jd_check.py
from .validate import verify


class JdCheck(object):
    """
    输入参数校验， 用于 web 请求参数校验
    """
    @staticmethod
    def verify(obj, **kwargs):  # 参数校验， 用法说明，请参考 verify 函数
        return verify(obj, **kwargs)
